@@ -18,18 +18,10 @@
  */
 
 /***************************************************************************
- * Copyright (C) 2017-2020 ZmartZone IAM
+ * Copyright (C) 2017-2021 ZmartZone Holding BV
  * Copyright (C) 2013-2017 Ping Identity Corporation
  * All rights reserved.
  *
- * For further information please contact:
- *
- *      Ping Identity Corporation
- *      1099 18th St Suite 2950
- *      Denver, CO 80202
- *      303.468.2900
- *      http://www.pingidentity.com
- *
  * DISCLAIMER OF WARRANTIES:
  *
  * THE SOFTWARE PROVIDED HEREUNDER IS PROVIDED ON AN "AS IS" BASIS, WITHOUT
@@ -67,7 +59,7 @@ apr_byte_t oidc_oauth_metadata_provider_retrieve(request_rec *r, oidc_cfg *cfg,
 	if (oidc_util_http_get(r, url, NULL, NULL, NULL,
 			cfg->oauth.ssl_validate_server, response, cfg->http_timeout_short,
 			cfg->outgoing_proxy, oidc_dir_cfg_pass_cookies(r),
-			NULL, NULL) == FALSE)
+			NULL, NULL, NULL) == FALSE)
 		return FALSE;
 
 	/* decode and see if it is not an error response somehow */
@@ -177,17 +169,28 @@ static apr_byte_t oidc_oauth_validate_access_token(request_rec *r, oidc_cfg *c,
 	return apr_strnatcmp(c->oauth.introspection_endpoint_method,
 			OIDC_INTROSPECTION_METHOD_GET) == 0 ?
 					oidc_util_http_get(r, c->oauth.introspection_endpoint_url, params,
-							basic_auth, bearer_auth, c->oauth.ssl_validate_server, response,
+							basic_auth, bearer_auth, c->oauth.ssl_validate_server,
+							response, c->http_timeout_long, c->outgoing_proxy,
+							oidc_dir_cfg_pass_cookies(r),
+							oidc_util_get_full_path(r->pool,
+									c->oauth.introspection_endpoint_tls_client_cert),
+							oidc_util_get_full_path(r->pool,
+									c->oauth.introspection_endpoint_tls_client_key),
+							oidc_util_get_full_path(r->pool,
+									c->oauth.introspection_endpoint_tls_client_key_pwd)
+					) :
+					oidc_util_http_post_form(r, c->oauth.introspection_endpoint_url,
+							params, basic_auth, bearer_auth,
+							c->oauth.ssl_validate_server, response,
 							c->http_timeout_long, c->outgoing_proxy,
 							oidc_dir_cfg_pass_cookies(r),
-							oidc_util_get_full_path(r->pool, c->oauth.introspection_endpoint_tls_client_cert),
-							oidc_util_get_full_path(r->pool, c->oauth.introspection_endpoint_tls_client_key)) :
-							oidc_util_http_post_form(r, c->oauth.introspection_endpoint_url,
-									params, basic_auth, bearer_auth, c->oauth.ssl_validate_server,
-									response, c->http_timeout_long, c->outgoing_proxy,
-									oidc_dir_cfg_pass_cookies(r),
-									oidc_util_get_full_path(r->pool, c->oauth.introspection_endpoint_tls_client_cert),
-									oidc_util_get_full_path(r->pool, c->oauth.introspection_endpoint_tls_client_key));
+							oidc_util_get_full_path(r->pool,
+									c->oauth.introspection_endpoint_tls_client_cert),
+							oidc_util_get_full_path(r->pool,
+									c->oauth.introspection_endpoint_tls_client_key),
+							oidc_util_get_full_path(r->pool,
+									c->oauth.introspection_endpoint_tls_client_key_pwd)
+					);
 }
 
 /*
@@ -301,35 +304,6 @@ apr_byte_t oidc_oauth_get_bearer_token(request_rec *r,
 	return TRUE;
 }
 
-/*
- * copy over space separated scope value but do it in an array for authorization purposes
- */
-/*
-static void oidc_oauth_spaced_string_to_array(request_rec *r, json_t *src,
-		const char *src_key, json_t *dst, const char *dst_key) {
-	apr_hash_t *ht = NULL;
-	apr_hash_index_t *hi = NULL;
-	json_t *arr = NULL;
-
-	json_t *src_val = json_object_get(src, src_key);
-
-	if (src_val != NULL)
-		ht = oidc_util_spaced_string_to_hashtable(r->pool,
-				json_string_value(src_val));
-
-	if (ht != NULL) {
-		arr = json_array();
-		for (hi = apr_hash_first(NULL, ht); hi; hi = apr_hash_next(hi)) {
-			const char *k;
-			const char *v;
-			apr_hash_this(hi, (const void**) &k, NULL, (void**) &v);
-			json_array_append_new(arr, json_string(v));
-		}
-		json_object_set_new(dst, dst_key, arr);
-	}
-}
-*/
-
 /*
  * parse (custom/configurable) token expiry claim in introspection result
  */
@@ -389,6 +363,13 @@ static apr_byte_t oidc_oauth_parse_and_cache_token_expiry(request_rec *r,
 static apr_byte_t oidc_oauth_cache_access_token(request_rec *r, oidc_cfg *c,
 		apr_time_t cache_until, const char *access_token, json_t *json) {
 
+	/* no cache mode */
+	int token_introspection_interval = oidc_cfg_token_introspection_interval(r);
+	if (token_introspection_interval == -1) {
+		oidc_debug(r, "not caching introspection result");
+		return TRUE;
+	}
+
 	oidc_debug(r, "caching introspection result");
 
 	json_t *cache_entry = json_object();
@@ -411,6 +392,12 @@ static apr_byte_t oidc_oauth_get_cached_access_token(request_rec *r,
 	json_t *cache_entry = NULL;
 	char *s_cache_entry = NULL;
 
+	/* no cache mode */
+	int token_introspection_interval = oidc_cfg_token_introspection_interval(r);
+	if (token_introspection_interval == -1) {
+		return FALSE;
+	}
+
 	/* see if we've got the claims for this access_token cached already */
 	oidc_cache_get_access_token(r, access_token, &s_cache_entry);
 
@@ -426,7 +413,6 @@ static apr_byte_t oidc_oauth_get_cached_access_token(request_rec *r,
 	/* compare the timestamp against the freshness requirement */
 	json_t *v = json_object_get(cache_entry, OIDC_OAUTH_CACHE_KEY_TIMESTAMP);
 	apr_time_t now = apr_time_sec(apr_time_now());
-	int token_introspection_interval = oidc_cfg_token_introspection_interval(r);
 	if ((token_introspection_interval > 0)
 			&& (now > json_integer_value(v) + token_introspection_interval)) {
 
@@ -641,7 +627,7 @@ static apr_byte_t oidc_oauth_validate_jwt_access_token(request_rec *r,
 	oidc_jwks_uri_t jwks_uri = { c->oauth.verify_jwks_uri,
 			c->provider.jwks_refresh_interval, c->oauth.ssl_validate_server };
 	if (oidc_proto_jwt_verify(r, c, jwt, &jwks_uri,
-			oidc_util_merge_key_sets(r->pool, c->oauth.verify_public_keys,
+			oidc_util_merge_key_sets_hash(r->pool, c->oauth.verify_public_keys,
 					c->oauth.verify_shared_keys), NULL) == FALSE) {
 		oidc_error(r,
 				"JWT access token signature could not be validated, aborting");
@@ -807,7 +793,7 @@ int oidc_oauth_check_userid(request_rec *r, oidc_cfg *c,
 
 	/* store the parsed token (cq. the claims from the response) in the request state so it can be accessed by the authz routines */
 	oidc_request_state_set(r, OIDC_REQUEST_STATE_KEY_CLAIMS,
-			(const char *) s_token);
+			(const char*) s_token);
 
 	/* set the request user */
 	if (oidc_oauth_set_request_user(r, c, token) == FALSE) {
@@ -828,18 +814,19 @@ int oidc_oauth_check_userid(request_rec *r, oidc_cfg *c,
 	char *authn_header = oidc_cfg_dir_authn_header(r);
 	apr_byte_t pass_headers = oidc_cfg_dir_pass_info_in_headers(r);
 	apr_byte_t pass_envvars = oidc_cfg_dir_pass_info_in_envvars(r);
+	apr_byte_t pass_base64url = oidc_cfg_dir_pass_info_base64url(r);
 
 	if ((r->user != NULL) && (authn_header != NULL))
 		oidc_util_hdr_in_set(r, authn_header, r->user);
 
 	/* set the resolved claims in the HTTP headers for the target application */
 	oidc_util_set_app_infos(r, token, oidc_cfg_claim_prefix(r),
-			c->claim_delimiter, pass_headers, pass_envvars);
+			c->claim_delimiter, pass_headers, pass_envvars, pass_base64url);
 
 	/* set the access_token in the app headers */
 	if (access_token != NULL) {
 		oidc_util_set_app_info(r, OIDC_APP_INFO_ACCESS_TOKEN, access_token,
-				OIDC_DEFAULT_HEADER_PREFIX, pass_headers, pass_envvars);
+				OIDC_DEFAULT_HEADER_PREFIX, pass_headers, pass_envvars, pass_base64url);
 	}
 
 	/* free JSON resources */
