@@ -18,18 +18,10 @@
  */
 
 /***************************************************************************
- * Copyright (C) 2017-2020 ZmartZone IAM
+ * Copyright (C) 2017-2021 ZmartZone Holding BV
  * Copyright (C) 2013-2017 Ping Identity Corporation
  * All rights reserved.
  *
- * For further information please contact:
- *
- *      Ping Identity Corporation
- *      1099 18th St Suite 2950
- *      Denver, CO 80202
- *      303.468.2900
- *      http://www.pingidentity.com
- *
  * DISCLAIMER OF WARRANTIES:
  *
  * THE SOFTWARE PROVIDED HEREUNDER IS PROVIDED ON AN "AS IS" BASIS, WITHOUT
@@ -1195,6 +1187,9 @@ const char *oidc_parse_info_hook_data(apr_pool_t *pool, const char *arg,
 			OIDC_HOOK_INFO_ID_TOKEN,
 			OIDC_HOOK_INFO_USER_INFO,
 			OIDC_HOOK_INFO_REFRESH_TOKEN,
+			OIDC_HOOK_INFO_SESSION_EXP,
+			OIDC_HOOK_INFO_SESSION_TIMEOUT,
+			OIDC_HOOK_INFO_SESSION_REMOTE_USER,
 			OIDC_HOOK_INFO_SESSION,
 			NULL };
 	const char *rv = oidc_valid_string_option(pool, arg, options);
@@ -1356,4 +1351,4 @@ const char *oidc_parse_set_state_input_headers_as(apr_pool_t *pool, const char *
 	}
 
 	return NULL;
-}
\ No newline at end of file
+}
